var LiveScript = require("LiveScript");

function compileAndInsertLiveCode(elem) {
  removeCodeResults(elem);
  appendResultContainers(elem);

  var jsCode = lsToJsFrom(elem);

  showJsCode(elem, jsCode);
  evalJsCode(elem, jsCode);
}

function lsToJsFrom(elem) {
  try {
    return LiveScript.compile($(elem).text(), {bare: true});
  } catch(e) {
    showError(elem, 'Compile', e);
  }
}

function removeCodeResults(elem) {
  $(elem).closest('section').find('.code_result').remove();
}

function appendResultContainers(elem) {
  $(elem).closest('section')
      .append(jsCompileResultDiv())
      .append(jsRunLogDiv());
}

function evalJsCode(elem, jsCode) {
  var orig_log = console.log;

  l = console.log = function() {
    var args = Array.prototype.slice.call(arguments);
    var $section = $(elem).closest('section');
    var $runLogCodeElem = $(elem).closest('section').find('.run_log code');
    $section.find('.run_log').show();
    args.forEach(function(arg) {$runLogCodeElem.append(arg);});
    $runLogCodeElem.append("\n");
  };

  d = function() {
    var args = Array.prototype.slice.call(arguments);
    l(args.map(function(arg) {return JSON.stringify(arg)}));
  }

  try {
    evalResult = eval(jsCode);
    showResult(elem, JSON.stringify(evalResult));
  } catch(e) {
    showError(elem, 'Runtime', e);
  }

  console.log = orig_log;
}

function showJsCode(elem, code) {
  $(elem).closest('section').find('.js_result code').html(code.substr("// Generated by LiveScript 1.2.0\n".length));
}

function jsCompileResultDiv(jsCode) {
  return '<div class="code_result js_result">' +
      '<h3>JavaScript</h3><pre><code data-trim ></code></pre>' +
    '</div>';
}

function jsRunLogDiv() {
  return '<div class="code_result run_log">' +
      '<h3>Run Log</h3><pre><code data-trim></code></pre>' +
    '</div>';
}

function showResult(elem, result)  {
  if (result === "" || result === undefined) return;
  var jsRunResultDiv =
    '<div class="code_result run_result"><h3>Result</h3>' +
      '<pre><code data-trim>' + result + '</code></pre>' +
    '</div>';
  $(elem).closest('section').append($(jsRunResultDiv));
}

function showError(elem, errorType, errorMessage) {
  var $section = $(elem).closest('section');
  $section.find('.run_log').show();
  $(elem).closest('section').find('.run_log h3').html(errorType + ' Error').css({color: 'red'});
  $(elem).closest('section').find('.run_log code').append(errorMessage).css({color: 'red'});
}
